"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const tesla_service_1 = require("../tesla/tesla.service");
const enphase_data_service_1 = require("./enphase.data.service");
const axios_1 = __importDefault(require("axios"));
const https = require("https");
const configuration_service_1 = require("../services/configuration.service");
// getting a production
const getProduction = (req, res, next) => __awaiter(void 0, void 0, void 0, function* () {
    var _a;
    let result = [];
    //try {
    if (!req.query.begin) {
        return res.status(400).json({ error: 'param begin not found' });
    }
    if (!req.query.end) {
        return res.status(400).json({ error: 'param end not found' });
    }
    if (!req.query.step) {
        return res.status(400).json({ error: 'param step not found' });
    }
    if (req.query.begin === 'start') {
        req.query.begin = '' + enphase_data_service_1.EnphaseDataService.getStartTimestamp();
    }
    // get timestamp
    let begin = Number(req.query.begin);
    let end = Number(req.query.end);
    let step = Number(req.query.step);
    let withCharge = ((((_a = req === null || req === void 0 ? void 0 : req.query) === null || _a === void 0 ? void 0 : _a.withCharge) + '').toLowerCase() === 'true');
    if (((end - begin) / step) > 5000 || begin >= end) {
        return res.status(400).json({ error: 'bad parameter values (more than 5000 step or begin>=end)' });
    }
    if (step < 1) {
        step = 1;
    }
    result = enphase_data_service_1.EnphaseDataService.getSteps(begin, end, step);
    if (withCharge) {
        result = tesla_service_1.TeslaService.addChargeValue(result);
    }
    //} catch (e) {
    //    return res.status(500).json( {error: 'can\'t get data'} );
    //}
    return res.status(200).json(result);
});
// getting a production
const getProductionForStep = (req, res, next) => __awaiter(void 0, void 0, void 0, function* () {
    let result = [];
    try {
        if (!req.query.steps) {
            return res.status(400).json({ error: 'param step not found' });
        }
        // get steps
        let steps = JSON.parse(req.query.steps);
        if (steps.length < 1) {
            return res.status(400).json({ error: 'bad steps parameter values' });
        }
        result = enphase_data_service_1.EnphaseDataService.getStepsValues(steps);
    }
    catch (e) {
        return res.status(500).json({ error: 'can\'t get data' });
    }
    return res.status(200).json(result);
});
// getting production real time
const getProductionRealTime = (req, res, next) => __awaiter(void 0, void 0, void 0, function* () {
    let data = {};
    try {
        data = enphase_data_service_1.EnphaseDataService.getProductionRealTime();
    }
    catch (e) {
        return res.status(500).json({ error: 'can\'t get data' });
    }
    return res.status(200).json(data);
});
// getting production real time one hour
const getProductionRealTimeOneHour = (req, res, next) => __awaiter(void 0, void 0, void 0, function* () {
    let data = {};
    try {
        data = enphase_data_service_1.EnphaseDataService.getProductionRealTimeOneHour();
    }
    catch (e) {
        return res.status(500).json({ error: 'can\'t get data' });
    }
    return res.status(200).json(data);
});
// getting production inverter
const getProductionInverter = (req, res, next) => __awaiter(void 0, void 0, void 0, function* () {
    let data = [];
    try {
        let result = yield enphase_data_service_1.EnphaseDataService.getProductionInverter();
        data = result.data;
    }
    catch (e) {
        return res.status(500).json({ error: 'can\'t get data (' + e + ')' });
    }
    return res.status(200).json(data);
});
const testGateway = (req, res, next) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        let response;
        if (configuration_service_1.ConfigurationService.getConfiguration().enphase.version === '5') {
            response = yield axios_1.default.get('http://' + configuration_service_1.ConfigurationService.getConfiguration().enphase.gateway + '/production.json', { 'headers': { 'Content-Type': 'application/json' } }).catch((error) => { res.status(error.status ? error.status : 500).send(error.message); });
        }
        else {
            const agent = new https.Agent({
                rejectUnauthorized: false
            });
            response = yield axios_1.default.get('https://' + configuration_service_1.ConfigurationService.getConfiguration().enphase.gateway + '/auth/check_jwt', { 'headers': { 'Authorization': 'Bearer ' + configuration_service_1.ConfigurationService.getConfiguration().enphase.token, 'Content-Type': 'application/json' }, httpsAgent: agent });
        }
        res.sendStatus(200);
    }
    catch (error) {
        try {
            res.status(500).send(error);
        }
        catch (error2) {
            console.error('Unable to send response from test gateway');
        }
    }
});
const postImportFile = (req, res, next) => __awaiter(void 0, void 0, void 0, function* () {
    var _b;
    // get the data from req.body
    let data = req.body;
    let confirm = ((((_b = req === null || req === void 0 ? void 0 : req.query) === null || _b === void 0 ? void 0 : _b.confirm) + '').toLowerCase() === 'true');
    let result = yield enphase_data_service_1.EnphaseDataService.importFile(data.data, confirm);
    // return response
    return res.status(200).json({ msg: result });
});
// getting a production and consumption for price
const getImportedForPrice = (req, res, next) => __awaiter(void 0, void 0, void 0, function* () {
    if (!req.query.begin) {
        return res.status(400).json({ error: 'param begin not found' });
    }
    if (!req.query.end) {
        return res.status(400).json({ error: 'param end not found' });
    }
    // get timestamp
    let begin = Number(req.query.begin);
    let end = Number(req.query.end);
    // Tarif base
    let result = enphase_data_service_1.EnphaseDataService.getBetweenDate(begin, end);
    let wh_imported = result.wh_consumed - result.wh_producted + result.wh_exported;
    return res.status(200).json({ wh_producted: result.wh_producted, wh_consumed: result.wh_consumed, wh_exported: result.wh_exported, wh_imported: wh_imported });
});
// getting a production and consumption for price
const getImportedForPrices = (req, res, next) => __awaiter(void 0, void 0, void 0, function* () {
    if (!req.query.dates) {
        return res.status(400).json({ error: 'param dates not found' });
    }
    let response = [];
    let dates = JSON.parse(req.query.dates);
    for (let i = 0; i < dates.length; i++) {
        let result = enphase_data_service_1.EnphaseDataService.getBetweenDate(dates[i].begin, dates[i].end);
        let wh_imported = result.wh_consumed - result.wh_producted + result.wh_exported;
        response.push({ wh_producted: result.wh_producted, wh_consumed: result.wh_consumed, wh_exported: result.wh_exported, wh_imported: wh_imported, data: dates[i].data });
    }
    return res.status(200).json(response);
});
exports.default = {
    postImportFile,
    getProduction,
    getProductionForStep,
    getProductionRealTime,
    getProductionRealTimeOneHour,
    getProductionInverter,
    getImportedForPrice,
    getImportedForPrices,
    testGateway
};
