"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
/** source/routes/posts.ts */
const express_1 = __importDefault(require("express"));
const posts_1 = __importDefault(require("../controllers/posts"));
const enphase_controller_1 = __importDefault(require("../enphase/enphase.controller"));
const tesla_controller_1 = __importDefault(require("../tesla/tesla.controller"));
const web_controller_1 = __importDefault(require("../web/web.controller"));
const router = express_1.default.Router();
router.get('/production', enphase_controller_1.default.getProduction);
router.get('/productionForStep', enphase_controller_1.default.getProductionForStep);
router.get('/productionRealTime', enphase_controller_1.default.getProductionRealTime);
router.get('/productionRealTimeOneHour', enphase_controller_1.default.getProductionRealTimeOneHour);
router.get('/productionInverter', enphase_controller_1.default.getProductionInverter);
router.get('/importedForPrice', enphase_controller_1.default.getImportedForPrice);
router.get('/importedForPrices', enphase_controller_1.default.getImportedForPrices);
router.get('/testGateway', enphase_controller_1.default.testGateway);
router.get('/teslaData', tesla_controller_1.default.getLastData);
router.get('/teslaRefresh', tesla_controller_1.default.getRefresh);
router.get('/teslaMode', tesla_controller_1.default.getMode);
router.get('/teslaClimate', tesla_controller_1.default.getClimate);
router.get('/teslaCharge', tesla_controller_1.default.getCharge);
router.post('/importFile', enphase_controller_1.default.postImportFile);
router.post('/saveConf', web_controller_1.default.postConf);
router.get('/checkPassword', web_controller_1.default.checkPassword);
router.get('*', web_controller_1.default.get);
router.get('/posts', posts_1.default.getPosts);
router.get('/posts/:id', posts_1.default.getPost);
router.put('/posts/:id', posts_1.default.updatePost);
router.delete('/posts/:id', posts_1.default.deletePost);
router.post('/posts', posts_1.default.addPost);
router.get('/api/tesla/status', (req, res) => {
  const teslaService = require('../tesla/tesla.service').TeslaService;
  
  // Récupérer les données MQTT
  const mqttClient = require('../tesla/tesla.mqtt-client');
  const mqttData = mqttClient?.getVehicleData('1');
  
  // Récupérer les meilleures données disponibles
  const bestData = teslaService.best_vehicle_data || teslaService.getLastReceiveData();
  
  // Déterminer quelle source utiliser (priorité à MQTT si valide)
  let sourceData;
  let dataSource;
  
  if (mqttData && mqttData.charge_state && mqttData.charge_state.battery_level > 0) {
    sourceData = mqttData;
    dataSource = 'mqtt';
    console.log("STATUS API: Utilisation données MQTT (temps réel)");
    
    // Mettre à jour les meilleures données connues avec les données MQTT
    teslaService.best_vehicle_data = mqttData;
  } 
  else if (bestData && bestData.charge_state && bestData.charge_state.battery_level > 0) {
    sourceData = bestData;
    dataSource = 'best_known';
    console.log("STATUS API: Utilisation des meilleures données connues");
  }
  else {
    // Fallback aux données API brutes
    sourceData = teslaService.getLastReceiveData() || {};
    dataSource = 'api_fallback';
    console.log("STATUS API: Données limitées disponibles");
  }
  
  // Ne jamais envoyer de réponse avec batterie à 0 si on a eu des meilleures données avant
  if (sourceData.charge_state?.battery_level === 0 && teslaService.best_vehicle_data) {
    console.log("STATUS API: Remplacement des données vides par les meilleures connues");
    sourceData = teslaService.best_vehicle_data;
    dataSource = 'best_cached';
  }
  
  // Formater la réponse pour le client
  res.json({
    battery_level: sourceData.charge_state?.battery_level || 0,
    charging_state: sourceData.charge_state?.charging_state || "Unknown",
    range_km: sourceData.charge_state?.est_battery_range_km || 0,
    inside_temp: sourceData.climate_state?.inside_temp || 0,
    outside_temp: sourceData.climate_state?.outside_temp || 0,
    is_climate_on: sourceData.climate_state?.is_climate_on || false,
    is_preconditioning: sourceData.climate_state?.is_preconditioning || false,
    data_source: dataSource,
    timestamp: Date.now(),
    display_name: sourceData.display_name || "Tesla",
    chargingMode: teslaService.chargingMode
  });
});

module.exports = router;
