"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
/** source/server.ts */
const http_1 = __importDefault(require("http"));
const express_1 = __importDefault(require("express"));
const morgan_1 = __importDefault(require("morgan"));
const router_1 = __importDefault(require("./routes/router"));
const enphase_data_service_1 = require("./enphase/enphase.data.service");
const path_1 = __importDefault(require("path"));
const tesla_service_1 = require("./tesla/tesla.service");
const configuration_service_1 = require("./services/configuration.service");
const socket_io_1 = require("socket.io");
const router = (0, express_1.default)();

// CORS
router.use((req, res, next) => {
    res.header('Access-Control-Allow-Origin', '*');
    res.header('Access-Control-Allow-Headers', 'origin, X-Requested-With,Content-Type,Accept, Authorization');
    if (req.method === 'OPTIONS') {
        res.header('Access-Control-Allow-Methods', 'GET POST');
        return res.status(200).json({});
    }
    next();
});

// Logging
router.use((0, morgan_1.default)('dev'));

// Parse
router.use(express_1.default.urlencoded({ extended: false }));
router.use(express_1.default.json());

// Static files
router.use('/assets', express_1.default.static(path_1.default.join(__dirname, 'assets')));
router.use(express_1.default.static(path_1.default.join(__dirname, 'webapp')));

// Routes
const teslaRoutes = require('./routes/tesla.routes');
router.use('/api/tesla', teslaRoutes);
router.use('/', router_1.default);

// Error handling
router.use((req, res, next) => {
    const error = new Error('not found');
    return res.status(404).json({
        message: error.message
    });
});

// Créer le serveur HTTP avant de configurer Socket.IO
const httpServer = http_1.default.createServer(router);

// Configurer Socket.IO
const io = new socket_io_1.Server(httpServer, {
  cors: {
    origin: "*",
    methods: ["GET", "POST"]
  }
});

// Rendre io disponible globalement
global.io = io;

// Gérer les connexions Socket.IO
io.on('connection', (socket) => {
  console.log('Client connecté:', socket.id);
  
  // Récupérer les données Tesla avec priorité MQTT
  const teslaService = require('./tesla/tesla.service').TeslaService;
  const mqttClient = require('./tesla/tesla.mqtt-client');
  
  // Priorité aux données MQTT si disponibles
  let teslaData;
  const mqttData = mqttClient?.getVehicleData('1');
  
  if (mqttData?.charge_state?.battery_level > 0) {
    teslaData = mqttData;
    console.log("SOCKET: Utilisation données MQTT à la connexion");
  } else {
    teslaData = teslaService.getLastReceiveData();
    console.log("SOCKET: Utilisation données API à la connexion");
  }
  
  // Envoyer les données immédiatement à la connexion
  if (teslaData && teslaData.charge_state) {
    // Format pour tesla-update (ancien format)
    socket.emit('tesla-update', {
      battery_level: teslaData.charge_state.battery_level,
      charging_state: teslaData.charge_state.charging_state,
      range_km: teslaData.charge_state.est_battery_range_km || 0,
      inside_temp: teslaData.climate_state?.inside_temp || 0,
      outside_temp: teslaData.climate_state?.outside_temp || 0,
      is_climate_on: teslaData.climate_state?.is_climate_on || false,
      is_preconditioning: teslaData.climate_state?.is_preconditioning || false,
      data_source: mqttData ? 'mqtt' : 'api',
      timestamp: Date.now(),
      display_name: teslaData.display_name || "Tesla",
      charger_actual_current: teslaData.charge_state?.charger_actual_current || 0,
      charger_voltage: teslaData.charge_state?.charger_voltage || 0,
      charger_power: teslaData.charge_state?.charger_power || 0,
      charging_mode: teslaService.chargingMode || "normal"  // Ajouter cette ligne
    });
    
    // Envoyer les données COMPLÈTES pour tesla:vehicle_update
    socket.emit('tesla:vehicle_update', teslaData);
  }
  
  // Écouter les demandes de rafraîchissement
  socket.on('refresh', () => {
    console.log('Demande de rafraîchissement Tesla reçue');
    // Demander un rafraîchissement des données à TeslaService
    teslaService.getImmediateVehicleData();
  });
  
  socket.on('tesla:refresh', () => {
    console.log('Demande de rafraîchissement Tesla:refresh reçue');
    teslaService.getImmediateVehicleData();
  });
  
  socket.on('disconnect', () => {
    console.log('Client déconnecté:', socket.id);
  });
});

// Propagation des événements Tesla - APRÈS l'initialisation de io
console.log("Configuration de la propagation d'événements Tesla...");

// Vérifier périodiquement les données Tesla et les propager
setInterval(() => {
  // Récupérer les données Tesla avec priorité MQTT
  const teslaService = require('./tesla/tesla.service').TeslaService;
  const mqttClient = require('./tesla/tesla.mqtt-client');
  
  let teslaData;
  const mqttData = mqttClient?.getVehicleData('1');
  
  if (mqttData?.charge_state?.battery_level > 0) {
    teslaData = mqttData;
    console.log("SOCKET: Utilisation données MQTT périodiques");
  } else {
    teslaData = teslaService.getLastReceiveData();
    console.log("SOCKET: Utilisation données API périodiques");
  }
  
  if (teslaData && teslaData.charge_state && teslaData.charge_state.battery_level) {
    // Format pour l'ancien panneau
    const simplifiedData = {
      battery_level: teslaData.charge_state.battery_level,
      charging_state: teslaData.charge_state.charging_state,
      range_km: teslaData.charge_state.est_battery_range_km || 0,
      inside_temp: teslaData.climate_state?.inside_temp || 0,
      outside_temp: teslaData.climate_state?.outside_temp || 0,
      is_climate_on: teslaData.climate_state?.is_climate_on || false,
      is_preconditioning: teslaData.climate_state?.is_preconditioning || false,
      data_source: mqttData ? 'mqtt' : 'api',
      timestamp: Date.now(),
      display_name: teslaData.display_name || "Tesla",
      charger_actual_current: teslaData.charge_state?.charger_actual_current || 0,
      charger_voltage: teslaData.charge_state?.charger_voltage || 0,
      charger_power: teslaData.charge_state?.charger_power || 0,
      charging_mode: teslaService.chargingMode || "normal"  // Ajouter cette ligne
    };
    
    // Émettre l'événement simplifié pour notre panneau
    io.emit('tesla-update', simplifiedData);
    
    // Émettre l'événement avec les données COMPLÈTES
    io.emit('tesla:vehicle_update', teslaData);
  }
}, 10000); // Vérifier toutes les 10 secondes

run();
function run() {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        // Init
        yield configuration_service_1.ConfigurationService.init();
        yield enphase_data_service_1.EnphaseDataService.init();
        yield tesla_service_1.TeslaService.init();
        
        const PORT = (_a = process.env.PORT) !== null && _a !== void 0 ? _a : 8002;
        httpServer.listen(PORT, () => console.log(`Le serveur est en écoute sur le port ${PORT}\r\nVous pouvez maintenant lancer un navigateur avec l'url "localhost:${PORT}"`));
    });
}